//! Structures and traits for block manipulation
//!
//! See the [OSdev wiki](https://wiki.osdev.org/Ext2#What_is_a_Block.3F) for more informations

use anyhow::Result;

/// Specifies all the necessary functions for a object to manipulate blocks
pub trait Manipulation {
    /// Returns the size in bytes of a single block
    fn block_size(&self) -> usize;

    /// Returns the length in bytes of the full I/O stream
    fn len(&self) -> usize;

    /// Reads `length` blocks of data from the I/O stream into the given `buffer` starting at the given `offset` (in **blocks**)
    ///
    /// Returns the number of blocks read
    fn read(&mut self, buffer: &mut [u8], offset: usize, length: usize) -> Result<usize>;

    /// Writes `length` blocks of data from the given `buffer` into I/O stream starting at the given `offset` (in **blocks**)
    ///
    /// Returns the number of blocks written
    fn write(&mut self, buffer: &mut [u8], offset: usize, length: usize) -> Result<usize>;

    /// Flushes the output stream, ensuring all contents in intermediate buffers are fully written out
    fn flush(&mut self) -> Result<()>;
}
