//! Interface to manage storage devices and controllers

use alloc::sync::Arc;
use alloc::vec::Vec;

use spin::Mutex;

use super::block::Manipulation;

/// Represents storage devices such as hard drive, SSDs, etc.
pub trait Device: Manipulation {
    /// Returns the size of the device in blocks
    fn size_in_blocks(&self) -> usize;
}

/// Represents a storage controller
pub trait Controller {
    /// Returns the list of the [`Device`] controlled by this structure
    fn devices<'device>(&self) -> Vec<Arc<Mutex<dyn Device + Send + 'device>>>;
}
